/*
 *	ccryptgui, QT based graphical user interface to ccrypt
 *
 *	Copyright (C) 2011  Martin Hilscher
 *
 *	This program is free software; you can redistribute it and/or modify it under the
 *	terms of the GNU General Public License as published by theFree Software Foundation;
 *	either version 3 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this
 *	program; if not, see <http://www.gnu.org/licenses/>.
 *
 *	------------------------------------------------------------------
 *
 *	main.cpp -  main-function	
 */

#include <QtGui/QApplication>
#include <QtCore/QTranslator>
#include <QLocale>
#include "ccryptgui.h"

#ifdef WIN
  #include "windows.h"
#else //Linux
  #include "unistd.h"
#endif


int main(int argc, char *argv[])
{
     int i;
     int MrkLoadLang = 0;

     QApplication app(argc, argv);

     QTranslator translator;

     QLocale::Language lang = QLocale::system().language();

     char ThisAppPath[ MAX_PATH_LENGTH ];
     char BinPath[ MAX_PATH_LENGTH ];

#ifdef WIN
     // fetch program-path (Windows)
     DWORD len;
     if ( ( len = GetModuleFileNameA( NULL, ThisAppPath, MAX_PATH_LENGTH ) ) )
         ThisAppPath[ len ] = '\0';
#else
     // fetch program-path (Linux)
     char path[ 40 ];
     ssize_t len;
     snprintf( path, 40, "/proc/%d/exe", getpid() );
     if ( ( len = readlink( path, ThisAppPath, MAX_PATH_LENGTH ) ) != -1 )
        ThisAppPath[ len ] = '\0';
#endif

     else
        MrkLoadLang += LOADLANG_ERR_PROG_PATH;

     // eliminate bin/debug/release-path from ThisAppPath
     char* pos = strrchr( ThisAppPath, DIR_SLASH_CHAR );
     if ( pos )
     {
        *pos = '\0';
        pos = strrchr( ThisAppPath, DIR_SLASH_CHAR );
        if ( pos )
           strcpy( BinPath, pos+1 );
           *(pos+1) = '\0';
     }
     else
        MrkLoadLang += LOADLANG_ERR_ELIM_BIN_PATH;

     // Parameter-defaults
     int debugmode = NO_DEBUG_MODE;

     // Ask for parameters
     // note: the translator has to be created before the application's widgets
     for ( i = 1; i < argc; i++)
     {
        // debug-mode
        if ( !strcmp( argv[i], "-dbg" ) ) debugmode = DEBUG_MODE;

        // force language = german
        if ( !strcmp( argv[i], "-lde" ) ) 
        {
           lang = QLocale::German;
        }

        // force language = englisch
        if ( !strcmp( argv[i], "-len" ) ) 
        {
           lang = QLocale::English;
        }

        // force language = spanish
        if ( !strcmp( argv[i], "-les" ) ) 
        {
           lang = QLocale::Spanish;
        }
     }

     // language is English -> no translation neccessary, because widgets defined in English
     
     QByteArray TranslPath = ThisAppPath;
     TranslPath     += TR_FILE_PATH;

     if ( lang == QLocale::German )
     {
        if ( !( translator.load( TR_FILE_DE, TranslPath ) ) )
           MrkLoadLang += LOADLANG_ERR_TRANSL_FILE;
        else
        {
           app.installTranslator( &translator );
           MrkLoadLang += LOADLANG_DE;
        }
     }

     if ( lang == QLocale::Spanish )
     {
        if ( !( translator.load( TR_FILE_ES, TranslPath ) ) )
           MrkLoadLang += LOADLANG_ERR_TRANSL_FILE;
        else
        {
           app.installTranslator( &translator );
           MrkLoadLang += LOADLANG_ES;
        }
     }
     // here place for further languages


     // Create main class
     CCryptGui* ccryptgui = new CCryptGui;

     // load some values to main class
     ccryptgui->debug_mode  = debugmode;
     ccryptgui->thisAppPath = ThisAppPath;
     ccryptgui->binPath     = BinPath;
     ccryptgui->mrkLoadLang = MrkLoadLang;

     // Set argument list of "ccrypt"
     ccryptgui->set_cmd_args();

     // Open Mainwindow
     ccryptgui->mainwindow->show();

     // Send first messages
     ccryptgui->messages_from_main();


     return app.exec();
}

